# AUTOGENERATED! DO NOT EDIT! File to edit: ../../notebooks/02e_pds.lroc_index.ipynb.

# %% auto 0
__all__ = ['LROCIndex']

# %% ../../notebooks/02e_pds.lroc_index.ipynb 2
import warnings
from nbdev.showdoc import *

# %% ../../notebooks/02e_pds.lroc_index.ipynb 3
class LROCIndex:
    def __init__(self):
        warnings.warn("LROCIndex not implemented yet.")
