# AUTOGENERATED BY NBDEV! DO NOT EDIT!

__all__ = ["index", "modules", "custom_doc_links", "git_url"]

index = {"Config": "00_config.ipynb",
         "config": "00_config.ipynb",
         "logger": "04_hirise.ipynb",
         "nasa_date_format": "01_utils.ipynb",
         "nasa_dt_format": "01_utils.ipynb",
         "nasa_dt_format_with_ms": "01_utils.ipynb",
         "iso_date_format": "01_utils.ipynb",
         "iso_dt_format": "01_utils.ipynb",
         "iso_dt_format_with_ms": "01_utils.ipynb",
         "nasa_time_to_datetime": "01_utils.ipynb",
         "nasa_time_to_iso": "01_utils.ipynb",
         "iso_to_nasa_time": "01_utils.ipynb",
         "iso_to_nasa_datetime": "01_utils.ipynb",
         "replace_all_nasa_times": "01_utils.ipynb",
         "parse_http_date": "01_utils.ipynb",
         "get_remote_timestamp": "01_utils.ipynb",
         "check_url_exists": "01_utils.ipynb",
         "url_retrieve": "01_utils.ipynb",
         "have_internet": "01_utils.ipynb",
         "height_from_shadow": "01_utils.ipynb",
         "get_gdal_center_coords": "01_utils.ipynb",
         "file_variations": "01_utils.ipynb",
         "storage_root": "06_cassini_iss.ipynb",
         "dynamic_urls": "02a_pds.indexes.ipynb",
         "Index": "02a_pds.indexes.ipynb",
         "CTXIndex": "02b_pds.ctx_index.ipynb",
         "find_indexes": "02c_pds.apps.ipynb",
         "get_index": "02c_pds.apps.ipynb",
         "find_instruments": "02c_pds.apps.ipynb",
         "refresh_index": "02c_pds.apps.ipynb",
         "OPUSImageURL": "02d_pds.opusapi.ipynb",
         "OPUSObsID": "02d_pds.opusapi.ipynb",
         "OPUS": "02d_pds.opusapi.ipynb",
         "base_url": "10_spice.kernels.ipynb",
         "metadata_url": "02d_pds.opusapi.ipynb",
         "image_url": "02d_pds.opusapi.ipynb",
         "dic": "02d_pds.opusapi.ipynb",
         "LROCIndex": "02e_pds.lroc_index.ipynb",
         "IndexLabel": "02f_pds.utils.ipynb",
         "index_to_df": "02f_pds.utils.ipynb",
         "PVLColumn": "02f_pds.utils.ipynb",
         "decode_line": "02f_pds.utils.ipynb",
         "find_mixed_type_cols": "02f_pds.utils.ipynb",
         "fix_hirise_edrcumindex": "02f_pds.utils.ipynb",
         "baseurl": "04_hirise.ipynb",
         "edrindex": "03_ctx.ipynb",
         "catch_isis_error": "03_ctx.ipynb",
         "CTXEDR": "03_ctx.ipynb",
         "CTXEDRCollection": "03_ctx.ipynb",
         "rdrindex": "04_hirise.ipynb",
         "OBSID": "04_hirise.ipynb",
         "ProductPathfinder": "04_hirise.ipynb",
         "COLOR_PRODUCT": "04_hirise.ipynb",
         "RGB_NOMAP": "04_hirise.ipynb",
         "RGB_NOMAPCollection": "04_hirise.ipynb",
         "SOURCE_PRODUCT": "04_hirise.ipynb",
         "RED_PRODUCT": "04_hirise.ipynb",
         "IR_PRODUCT": "04_hirise.ipynb",
         "BG_PRODUCT": "04_hirise.ipynb",
         "DataManager": "05_cassini_uvis.ipynb",
         "get_data_path": "07_diviner.ipynb",
         "get_label_path": "05_cassini_uvis.ipynb",
         "get_user_guide": "05_cassini_uvis.ipynb",
         "opus_keys": "06_cassini_iss.ipynb",
         "ISSOpus": "06_cassini_iss.ipynb",
         "hostname": "07_diviner.ipynb",
         "calculate_image_azimuth": "08_geotools.ipynb",
         "get_north_shifted_point": "08_geotools.ipynb",
         "calculate_image_north_azimuth": "08_geotools.ipynb",
         "get_sun_angles": "08_geotools.ipynb",
         "debug_srs": "08_geotools.ipynb",
         "shift_to_center": "08_geotools.ipynb",
         "pixel_to_meter": "08_geotools.ipynb",
         "Point": "08_geotools.ipynb",
         "Window": "08_geotools.ipynb",
         "ImgData": "08_geotools.ipynb",
         "Error": "09_exceptions.ipynb",
         "SomethingNotSetError": "09_exceptions.ipynb",
         "ProjectionNotSetError": "09_exceptions.ipynb",
         "GeoTransformNotSetError": "09_exceptions.ipynb",
         "KMASpiceError": "09_exceptions.ipynb",
         "SPointNotSetError": "09_exceptions.ipynb",
         "ObserverNotSetError": "09_exceptions.ipynb",
         "kernel_storage": "10_spice.kernels.ipynb",
         "dataset_ids": "10_spice.kernels.ipynb",
         "df": "10_spice.kernels.ipynb",
         "df2": "10_spice.kernels.ipynb",
         "df2.columns": "10_spice.kernels.ipynb",
         "datasets": "10_spice.kernels.ipynb",
         "is_start_valid": "10_spice.kernels.ipynb",
         "is_stop_valid": "10_spice.kernels.ipynb",
         "Subsetter": "10_spice.kernels.ipynb",
         "get_metakernel_and_files": "10_spice.kernels.ipynb",
         "list_kernels_for_day": "10_spice.kernels.ipynb"}

modules = ["config.py",
           "utils.py",
           "pds/indexes.py",
           "pds/ctx_index.py",
           "pds/apps.py",
           "pds/opusapi.py",
           "pds/lroc_index.py",
           "pds/utils.py",
           "ctx.py",
           "hirise.py",
           "uvis.py",
           "ciss.py",
           "diviner.py",
           "geotools.py",
           "exceptions.py",
           "spice/kernels.py"]

doc_url = "https://michaelaye.github.io/nbplanetary/"

git_url = "https://github.com/michaelaye/nbplanetary/tree/master/"

def custom_doc_links(name): return None
